@@ -11,11 +11,12 @@
 #    under the License.
 
 import logging
+import shlex
 
 from django.conf import settings
 
 from horizon import exceptions
-from horizon.utils.memoized import memoized_with_request
+from horizon.utils.memoized import memoized
 from openstack_dashboard.api import base
 
 from neutronclient.v2_0 import client as neutron_client
@@ -51,22 +52,32 @@ def get_auth_params_from_request(request):
     )
 
 
-@memoized_with_request(get_auth_params_from_request)
-def zunclient(request_auth_params):
+@memoized
+def zunclient(request):
     (
         username,
         token_id,
         project_id,
         endpoint_override
-    ) = request_auth_params
+    ) = get_auth_params_from_request(request)
 
     LOG.debug('zunclient connection created using the token "%s" and url'
               ' "%s"' % (token_id, endpoint_override))
+    api_version = API_VERSION
+    if API_VERSION.is_latest():
+        c = zun_client.Client(
+            username=username,
+            project_id=project_id,
+            auth_token=token_id,
+            endpoint_override=endpoint_override,
+            api_version=api_versions.APIVersion("1.1"),
+        )
+        api_version = api_versions.discover_version(c, api_version)
     c = zun_client.Client(username=username,
                           project_id=project_id,
                           auth_token=token_id,
                           endpoint_override=endpoint_override,
-                          api_version=API_VERSION)
+                          api_version=api_version)
     return c
 
 
@@ -82,9 +93,13 @@ def get_auth_params_from_request_neutron(request):
     )
 
 
-@memoized_with_request(get_auth_params_from_request_neutron)
-def neutronclient(request_auth_params):
-    token_id, neutron_url, auth_url = request_auth_params
+@memoized
+def neutronclient(request):
+    (
+        token_id,
+        neutron_url,
+        auth_url
+    ) = get_auth_params_from_request_neutron(request)
     insecure = getattr(settings, 'OPENSTACK_SSL_NO_VERIFY', False)
     cacert = getattr(settings, 'OPENSTACK_SSL_CACERT', None)
 
@@ -121,6 +136,8 @@ def _cleanup_params(attrs, check, **params):
                 kv = v.split("=", 1)
                 values[kv[0]] = kv[1]
             args[str(key)] = values
+        elif key == "command":
+            args[key] = shlex.split(value)
         elif key in attrs:
             if value is None:
                 value = ''
@@ -184,9 +201,7 @@ def container_delete(request, **kwargs):
 
 
 def container_list(request, limit=None, marker=None, sort_key=None,
-                   sort_dir=None, detail=True):
-    # TODO(shu-mutou): detail option should be added, if it is
-    # implemented in Zun API
+                   sort_dir=None):
     return zunclient(request).containers.list(limit, marker, sort_key,
                                               sort_dir)
 
@@ -289,10 +304,9 @@ def capsule_delete(request, **kwargs):
 
 
 def image_list(request, limit=None, marker=None, sort_key=None,
-               sort_dir=None, detail=False):
-    # FIXME(shu-mutou): change "detail" param to True, if it enabled.
+               sort_dir=None):
     return zunclient(request).images.list(limit, marker, sort_key,
-                                          sort_dir, detail)
+                                          sort_dir)
 
 
 def image_create(request, **kwargs):
@@ -305,6 +319,10 @@ def image_delete(request, id, **kwargs):
 
 
 def host_list(request, limit=None, marker=None, sort_key=None,
-              sort_dir=None, detail=False):
+              sort_dir=None):
     return zunclient(request).hosts.list(limit, marker, sort_key,
-                                         sort_dir, detail)
+                                         sort_dir)
+
+
+def host_show(request, id):
+    return zunclient(request).hosts.get(id)
